==> An_V1.9.0.tar.gz: kmpdfkit_demo/Annotations/src/main/java/com/compdfkit/annotations/PDFAnnotationSampleActivity.java <==
package com.compdfkit.annotations;

import android.Manifest;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;

import com.compdfkit.annotations.CAnnotSampleScreenManager;
import com.compdfkit.annotations.R;
import com.compdfkit.annotations.databinding.AnnotationPdfSampleActivityBinding;
import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.tools.common.activity.CBasicActivity;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.dialog.CAlertDialog;
import com.compdfkit.tools.common.utils.task.CExtractAssetFileTask;
import com.compdfkit.tools.common.utils.window.CPopupMenuWindow;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaDialogFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfproperties.CAnnotationType;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.viewer.pdfsearch.CSearchResultDialogFragment;

import java.util.ArrayList;
import java.util.Arrays;

import pub.devrel.easypermissions.AppSettingsDialog;
import pub.devrel.easypermissions.EasyPermissions;

public class PDFAnnotationSampleActivity extends CBasicActivity {

    /**
     * assets folder pdf file
     */
    public static final String QUICK_START_GUIDE = "Quick Start Guide V2.4.0.pdf";

    private AnnotationPdfSampleActivityBinding binding;

    CAnnotSampleScreenManager screenManager = new CAnnotSampleScreenManager();

    private ActivityResultLauncher<Intent> selectDocumentLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result.getData() != null && result.getData().getData() != null) {
            Uri uri = result.getData().getData();
            CFileUtils.takeUriPermission(this, uri);
            binding.pdfView.resetAnnotationType();
            binding.pdfView.openPDF(uri);
        }
    });

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = AnnotationPdfSampleActivityBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        screenManager.bind(binding);
        //Extract PDF files from the Android assets folder
        initPDFView();
        initAnnotationAttributes();
        initToolBarView();
        initSearchBar();
        initAnnotToolbar();
    }

    private void initPDFView() {
        binding.pdfView.setAllowAddAndEditAnnot(true);
        CExtractAssetFileTask.extract(this, QUICK_START_GUIDE, QUICK_START_GUIDE, (filePath) ->
                binding.pdfView.openPDF(filePath));
        resetContextMenu(binding.pdfView, CPreviewMode.Annotation);
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
        binding.pdfView.addOnPDFFocusedTypeChangeListener(type -> {
            if (type != CPDFAnnotation.Type.INK) {
                if (binding.inkCtrlView.getVisibility() == View.VISIBLE) {
                    screenManager.changeWindowStatus(type);
                    binding.annotationToolBar.updateItemColor();
                }
            }
        });
        binding.pdfView.setOnTapMainDocAreaCallback(() -> {
            //Use the CFillScreenManager.class to manage fullscreen switching.
            screenManager.fillScreenChange();
        });
    }

    /**
     * Initialize and set annotation attributes,
     * mainly including annotation color, transparency and other attributes
     */
    private void initAnnotationAttributes() {
        initAnnotationAttr(binding.pdfView);
        registerAnnotHelper(binding.pdfView);
    }

    private void initToolBarView() {
        binding.pdfToolBar.addMode(CPreviewMode.Annotation);
        binding.pdfToolBar.setPreviewModeChangeListener((mode) -> {
            binding.pdfToolBar.selectMode(mode);
            screenManager.changeWindowStatus(mode);
            binding.pdfView.setAllowAddAndEditAnnot(mode != CPreviewMode.Viewer);
            binding.pdfView.resetAnnotationType();
            resetContextMenu(binding.pdfView, mode);
        });
        binding.pdfToolBar.setSearchBtnClickListener(v -> {
            binding.pdfToolBar.setVisibility(View.GONE);
            binding.pdfSearchToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.showKeyboard();
        });
        binding.pdfToolBar.setThumbnailBtnClickListener(v -> {
            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.THUMBNAIL, getString(R.string.tools_thumbnails));
            dialogFragment.setBotaDialogTab(annotationTab);
            dialogFragment.show(getSupportFragmentManager(), "ThumbnailList");
        });
        binding.pdfToolBar.setBoTaBtnClickListener(v -> {
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.ANNOTATION, getString(R.string.tools_annotations));
            CPDFBotaFragmentTabs outlineTab = new CPDFBotaFragmentTabs(CPDFBOTA.OUTLINE, getString(R.string.tools_outlines));
            CPDFBotaFragmentTabs bookmarkTab = new CPDFBotaFragmentTabs(CPDFBOTA.BOOKMARKS, getString(R.string.tools_bookmarks));
            if (binding.pdfToolBar.getMode() == CPreviewMode.Viewer) {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
            } else {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
                tabs.add(annotationTab);
            }
            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            dialogFragment.setBotaDialogTabs(tabs);
            dialogFragment.show(getSupportFragmentManager(), "annotationList");
        });
        binding.pdfToolBar.setMoreBtnClickListener(v -> {
            v.setSelected(true);
            //Show the PDF settings dialog fragment
            CPopupMenuWindow menuWindow = new CPopupMenuWindow(this);
            menuWindow.addItem(R.drawable.tools_ic_preview_settings, R.string.tools_view_setting, v1 -> {
                showDisplaySettings(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_document_info, R.string.tools_document_info, v1 -> {
                showDocumentInfo(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_share, R.string.tools_share, v1 -> {
                sharePDF(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_new_file, R.string.tools_open_document, v1 -> {
                if (hasPermissions(STORAGE_PERMISSIONS)) {
                    selectDocument();
                } else {
                    EasyPermissions.requestPermissions(this, "request permission", REQUEST_EXTERNAL_PERMISSION, STORAGE_PERMISSIONS);
                }
            });
            menuWindow.setOnDismissListener(() -> v.setSelected(false));
            menuWindow.showAsDropDown(v);
        });
    }

    private void initAnnotToolbar() {
        binding.annotationToolBar.initWithPDFView(binding.pdfView);
        binding.annotationToolBar.setFragmentManager(getSupportFragmentManager());
        binding.annotationToolBar.setAnnotationChangeListener(type -> {
            screenManager.changeWindowStatus(type);
            //You are required to grant recording permission when selecting voice notes
            if (type == CAnnotationType.SOUND) {
                if (!EasyPermissions.hasPermissions(this, Manifest.permission.RECORD_AUDIO)) {
                    EasyPermissions.requestPermissions(this, getString(R.string.tools_use_sound_annot), 111, Manifest.permission.RECORD_AUDIO);
                }
            }
        });
        binding.inkCtrlView.initWithPDFView(binding.pdfView);
        binding.inkCtrlView.setFragmentManager(getSupportFragmentManager());
    }

    private void initSearchBar() {
        binding.pdfSearchToolBar.initWithPDFView(binding.pdfView);
        binding.pdfSearchToolBar.onSearchQueryResults(list -> {
            CSearchResultDialogFragment searchResultDialog = new CSearchResultDialogFragment();
            searchResultDialog.show(getSupportFragmentManager(), "searchResultDialogFragment");
            searchResultDialog.setSearchTextInfos(list);
            searchResultDialog.setOnClickSearchItemListener(clickItem -> {
                binding.pdfView.getCPdfReaderView().setDisplayPageIndex(clickItem.page);
                binding.pdfView.getCPdfReaderView().getTextSearcher().searchBegin(clickItem.page, clickItem.textRangeIndex);
                searchResultDialog.dismiss();
            });
        });
        binding.pdfSearchToolBar.setExitSearchListener(() -> {
            binding.pdfToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.setVisibility(View.GONE);
        });
    }

    private void selectDocument() {
        if (!binding.pdfView.getCPdfReaderView().getPDFDocument().hasChanges()) {
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            return;
        }
        CAlertDialog alertDialog = CAlertDialog.newInstance(getString(com.compdfkit.tools.R.string.tools_save_title), getString(com.compdfkit.tools.R.string.tools_save_message));
        alertDialog.setConfirmClickListener(v -> {
            //save pdf document
            binding.pdfView.savePDF((filePath, pdfUri) -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            });
        });
        alertDialog.setCancelClickListener(v -> {
            alertDialog.dismiss();
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
        });
        alertDialog.show(getSupportFragmentManager(), "alertDialog");
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_EXTERNAL_PERMISSION) {
            if (EasyPermissions.hasPermissions(this, STORAGE_PERMISSIONS)) {
                selectDocument();
            }
        } else if (requestCode == 111) {
            if (!EasyPermissions.hasPermissions(this, permissions)) {
                if (EasyPermissions.somePermissionPermanentlyDenied(this, Arrays.asList(permissions))) {
                    new AppSettingsDialog.Builder(this)
                            .build().show();
                }else {
                    binding.pdfView.resetAnnotationType();
                }
            }
        } else if (requestCode == AppSettingsDialog.DEFAULT_SETTINGS_REQ_CODE){
            if (EasyPermissions.hasPermissions(this, permissions)){

            }
        }
    }

    @Override
    public void onBackPressed() {
        if (binding.pdfView != null) {
            binding.pdfView.savePDF((filePath, pdfUri) -> super.onBackPressed());
        }
    }
}

==> An_V2.0.3.tar.gz: kmpdfkit_demo/Annotations/src/main/java/com/compdfkit/annotations/PDFAnnotationSampleActivity.java <==
package com.compdfkit.annotations;

import android.Manifest;
import android.os.Build;
import android.os.Bundle;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;

import com.compdfkit.annotations.databinding.AnnotationPdfSampleActivityBinding;
import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.tools.common.basic.activity.CBasicPDFActivity;
import com.compdfkit.tools.common.pdf.CPDFApplyConfigUtil;
import com.compdfkit.tools.common.pdf.CPDFConfigurationUtils;
import com.compdfkit.tools.common.pdf.config.AnnotationsConfig;
import com.compdfkit.tools.common.pdf.config.CPDFConfiguration;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.CPermissionUtil;
import com.compdfkit.tools.common.utils.activitycontracts.CSelectPDFDocumentResultContract;
import com.compdfkit.tools.common.utils.dialog.CAlertDialog;
import com.compdfkit.tools.common.utils.task.CExtractAssetFileTask;
import com.compdfkit.tools.common.utils.window.CPopupMenuWindow;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaDialogFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfproperties.CAnnotationType;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.viewer.pdfsearch.CSearchResultDialogFragment;
import com.compdfkit.ui.reader.CPDFReaderView;

import java.util.ArrayList;
import java.util.Arrays;


public class PDFAnnotationSampleActivity extends CBasicPDFActivity {

    /**
     * assets folder pdf file
     */
    public static final String QUICK_START_GUIDE = "ComPDFKit_Annotations_Sample_File.pdf";

    private AnnotationPdfSampleActivityBinding binding;

    CAnnotSampleScreenManager screenManager = new CAnnotSampleScreenManager();

    private CPDFConfiguration cpdfConfiguration;

    private ActivityResultLauncher<Void> selectDocumentLauncher = registerForActivityResult(new CSelectPDFDocumentResultContract(), uri -> {
        if (uri != null) {
            CFileUtils.takeUriPermission(this, uri);
            binding.pdfView.resetAnnotationType();
            binding.pdfView.openPDF(uri);
        }
    });

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = AnnotationPdfSampleActivityBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        screenManager.bind(binding);
        //Extract PDF files from the Android assets folder
        initPDFView();
        initAnnotationAttributes();
        initToolBarView();
        initSearchBar();
        initAnnotToolbar();
    }

    private void initPDFView() {
        binding.pdfView.setViewMode(CPDFReaderView.ViewMode.ANNOT);
        CExtractAssetFileTask.extract(this, QUICK_START_GUIDE, QUICK_START_GUIDE, (filePath) ->
                binding.pdfView.openPDF(filePath));
        resetContextMenu(binding.pdfView, CPreviewMode.Annotation);
        registerFormHelper(binding.pdfView);
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
        binding.pdfView.addOnPDFFocusedTypeChangeListener(type -> {
            if (type != CPDFAnnotation.Type.INK) {
                if (binding.inkCtrlView.getVisibility() == View.VISIBLE) {
                    screenManager.changeWindowStatus(type);
                    binding.annotationToolBar.updateItemColor();
                }
            }
        });
        binding.pdfView.setOnTapMainDocAreaCallback(() -> {
            //Use the CFillScreenManager.class to manage fullscreen switching.
            screenManager.fillScreenChange();
        });
    }

    /**
     * Initialize and set annotation attributes,
     * mainly including annotation color, transparency and other attributes
     */
    private void initAnnotationAttributes() {
        initAnnotationAttr(binding.pdfView);
        registerAnnotHelper(binding.pdfView);
    }

    private void initToolBarView() {
        binding.pdfToolBar.addMode(CPreviewMode.Viewer);
        binding.pdfToolBar.addMode(CPreviewMode.Annotation);
        binding.pdfToolBar.selectMode(CPreviewMode.Annotation);
        binding.pdfToolBar.setPreviewModeChangeListener((mode) -> {
            binding.pdfToolBar.selectMode(mode);
            screenManager.changeWindowStatus(mode);
            if (mode == CPreviewMode.Viewer) {
                binding.pdfView.setViewMode(CPDFReaderView.ViewMode.VIEW);
            } else {
                binding.pdfView.setViewMode(CPDFReaderView.ViewMode.ANNOT);
            }
            binding.pdfView.resetAnnotationType();
            resetContextMenu(binding.pdfView, mode);
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_thumbnail, v -> {
            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.THUMBNAIL, getString(R.string.tools_thumbnails));
            dialogFragment.setBotaDialogTab(annotationTab);
            dialogFragment.show(getSupportFragmentManager(), "ThumbnailList");
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_search, v -> {
            binding.pdfToolBar.setVisibility(View.GONE);
            binding.pdfSearchToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.showKeyboard();
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_bookmark, v -> {
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.ANNOTATION, getString(R.string.tools_annotations));
            CPDFBotaFragmentTabs outlineTab = new CPDFBotaFragmentTabs(CPDFBOTA.OUTLINE, getString(R.string.tools_outlines));
            CPDFBotaFragmentTabs bookmarkTab = new CPDFBotaFragmentTabs(CPDFBOTA.BOOKMARKS, getString(R.string.tools_bookmarks));
            if (binding.pdfToolBar.getMode() == CPreviewMode.Viewer) {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
            } else {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
                tabs.add(annotationTab);
            }
            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            dialogFragment.setBotaDialogTabs(tabs);
            dialogFragment.show(getSupportFragmentManager(), "annotationList");
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_more, v -> {
            v.setSelected(true);
            //Show the PDF settings dialog fragment
            CPopupMenuWindow menuWindow = new CPopupMenuWindow(this);
            menuWindow.addItem(R.drawable.tools_ic_preview_settings, R.string.tools_view_setting, v1 -> {
                showDisplaySettings(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_document_info, R.string.tools_document_info, v1 -> {
                showDocumentInfo(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_share, R.string.tools_share, v1 -> {
                sharePDF(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_new_file, R.string.tools_open_document, v1 -> {
                if (CPermissionUtil.hasStoragePermissions(this)) {
                    selectDocument();
                } else {
                    requestStoragePermissions();
                }
            });
            menuWindow.setOnDismissListener(() -> v.setSelected(false));
            menuWindow.showAsDropDown(v);
        });
    }

    private void requestStoragePermissions() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            CPermissionUtil.openManageAllFileAppSettings(this);
        } else {
            multiplePermissionResultLauncher.launch(STORAGE_PERMISSIONS, result -> {
                if (CPermissionUtil.hasStoragePermissions(this)) {
                    selectDocument();
                }else {
                    if (!CPermissionUtil.shouldShowRequestPermissionRationale(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)){
                        showPermissionsRequiredDialog();
                    }
                }
            });
        }
    }

    private void initAnnotToolbar() {
        binding.annotationToolBar.initWithPDFView(binding.pdfView);
        binding.annotationToolBar.setTools(Arrays.asList(
                AnnotationsConfig.AnnotationTools.Setting,
                AnnotationsConfig.AnnotationTools.Undo,
                AnnotationsConfig.AnnotationTools.Redo
        ));
        binding.annotationToolBar.setFragmentManager(getSupportFragmentManager());
        binding.annotationToolBar.setAnnotationChangeListener(type -> {
            screenManager.changeWindowStatus(type);
            //You are required to grant recording permission when selecting voice notes
            if (type == CAnnotationType.SOUND) {
                if (!hasPermission(Manifest.permission.RECORD_AUDIO)) {
                    permissionResultLauncher.launch(Manifest.permission.RECORD_AUDIO, hasRecordAudioPermission -> {
                        if (!hasRecordAudioPermission){
                            binding.pdfView.resetAnnotationType();
                            if (!CPermissionUtil.shouldShowRequestPermissionRationale(this, Manifest.permission.RECORD_AUDIO)){
                                showPermissionsRequiredDialog();
                            }
                        }
                    });
                }
            }
        });
        binding.inkCtrlView.initWithPDFView(binding.pdfView);
        binding.inkCtrlView.setFragmentManager(getSupportFragmentManager());
    }



    private void initSearchBar() {
        binding.pdfSearchToolBar.initWithPDFView(binding.pdfView);
        binding.pdfSearchToolBar.onSearchQueryResults(list -> {
            CSearchResultDialogFragment searchResultDialog = new CSearchResultDialogFragment();
            searchResultDialog.show(getSupportFragmentManager(), "searchResultDialogFragment");
            searchResultDialog.setSearchTextInfos(list);
            searchResultDialog.setOnClickSearchItemListener(clickItem -> {
                binding.pdfView.getCPdfReaderView().setDisplayPageIndex(clickItem.page);
                binding.pdfView.getCPdfReaderView().getTextSearcher().searchBegin(clickItem.page, clickItem.textRangeIndex);
                searchResultDialog.dismiss();
            });
        });
        binding.pdfSearchToolBar.setExitSearchListener(() -> {
            binding.pdfToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.setVisibility(View.GONE);
        });
    }

    private void selectDocument() {
        if (binding.pdfView.getCPdfReaderView().getPDFDocument() == null){
            selectDocumentLauncher.launch(null);
            return;
        }
        if (!binding.pdfView.getCPdfReaderView().getPDFDocument().hasChanges()) {
            selectDocumentLauncher.launch(null);
            return;
        }
        CAlertDialog alertDialog = CAlertDialog.newInstance(getString(com.compdfkit.tools.R.string.tools_save_title), getString(com.compdfkit.tools.R.string.tools_save_message));
        alertDialog.setConfirmClickListener(v -> {
            //save pdf document
            binding.pdfView.savePDF((filePath, pdfUri) -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(null);
            }, e -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(null);
            });
        });
        alertDialog.setCancelClickListener(v -> {
            alertDialog.dismiss();
            selectDocumentLauncher.launch(null);
        });
        alertDialog.show(getSupportFragmentManager(), "alertDialog");
    }


    private void parseConfiguration() {
        if (cpdfConfiguration == null) {
            cpdfConfiguration = CPDFConfigurationUtils.normalConfig(this, "tools_default_configuration.json");
        }
    }

    @Override
    public void onBackPressed() {
        if (binding.pdfView != null) {
            binding.pdfView.savePDF((filePath, pdfUri) -> super.onBackPressed(), e -> super.onBackPressed());
        } else {
            super.onBackPressed();
        }
    }
}
